--

-- ADVance MS model for intelligent voltage-driven switch 
-- The switch operates as a finite-state automaton
--
LIBRARY IEEE,DISCIPLINES;
USE IEEE.math_real.ALL;
USE DISCIPLINES.ELECTROMAGNETIC_SYSTEM.ALL;


ENTITY key_limiting_adms_vhdlams_model IS
     GENERIC(
          v1:real:=0.0; -- low limit for the swith command
          v2:real:=10000.0 -- high limit for the swith command
          -- the default values for v1 and v2 are chosen so to
          -- desactivate the flyback (the Vstore voltage will 
          -- probably never reach 10000 Volts, so the switch is 
          -- never activated. 
            );
	PORT (
		TERMINAL ep       : Electrical ;
		TERMINAL em       : Electrical ;
		TERMINAL store    : Electrical ; 
		TERMINAL common   : Electrical 
	);
END ENTITY key_limiting_adms_vhdlams_model;

ARCHITECTURE adms_vhdlams OF key_limiting_adms_vhdlams_model IS

quantity v_store across store to common ;
quantity vsw across isw through ep to em ;

constant ron:real:=1.0e-9;
constant roff:real:=1.0e15; 

signal ison : boolean;

BEGIN


PROCESS
variable off: boolean :=true;

BEGIN

ison <= not off;
CASE off IS
	WHEN true => 
		WAIT until v_store'above(v2);	
		off:=false;
	WHEN false =>
		WAIT until NOT v_store'above(v1);
		off:=true;
	END CASE;
END PROCESS;

--*********************************************************

IF ison use isw==vsw/ron;
ELSE isw==vsw/roff;
END use;
BREAK ON ison;
	
END ARCHITECTURE adms_vhdlams;
