--
-- ADVance MS model for electromechanical harvesting_resonator adms_vhdlams_ESYCOM_juine2007
--
LIBRARY DISCIPLINES;
USE DISCIPLINES.ELECTROMAGNETIC_SYSTEM.ALL;
library IEEE;
use IEEE.MATH_REAL.all;



ENTITY electrostatic_transducer IS
	GENERIC(
          d:real:=1.0e-6; --transducer's gap 
          l:real:=50.0e-6; --transducer's length
          w:real:=100.0e-6; --transducer's width
		Q0:real:=0.0; --initial charge
		x0:real:=0.0 --initial position
		);

     -- les terminaux : C1 et C2 are the capacitor's electrodes 
     --                 Mp et Mm are the mechanical terminal
     --                 generating the transducer's force (voltage)  
     --                 Their polarity is important
     --                 pos: the x-coordinate of the transducer, 
     --                       an output termninal for measurement purposes
	PORT (
		TERMINAL C1        : Electrical ;
          TERMINAL C2      : Electrical ;
		TERMINAL Mp     : Electrical ;
		TERMINAL Mm         : Electrical; 
          TERMINAL pos          : Electrical
	);
END;

ARCHITECTURE gap_closing_transducer OF electrostatic_transducer IS

function transducer_capacitance_gap_closing(x:real; d:real;S:real) return real is
variable capa:real;
begin
	capa:=8.85e-12*S/(d-x);
return capa;
end;

quantity charge : real:=0.0;
quantity dCvar_dx : real;
quantity Cvar : real;
quantity uC across iC through C1 to C2; -- the capacitor voltage
quantity force across v through Mp to Mm; -- the force of transducer 
quantity x across ix through pos to electrical_ground; -- position of the mobile electrode of transducer 
constant dx :real:=0.001e-6;
BEGIN

-- mechanical side model : a force generator. 
-- However, the position must be known for the force calculation
-- We integrate the speed (the current through Mp and Mm) to know the position 
x==-v'integ+x0;
-- then we calculate the capacitance gradient  at x 
dCvar_dx==(transducer_capacitance_gap_closing(x+dx, d, l*w)-transducer_capacitance_gap_closing(x, d, l*w))/dx;
-- and the force
force==uC*uC*dCvar_dx/2.0;

-- Electrical side model : a simple capacitor
-- first we calculate the capacitance value at x
Cvar==transducer_capacitance_gap_closing(x, d, l*w);
-- then the charge 
charge==iC'integ+Q0;
-- then the capacitor voltage
uC==charge/Cvar;

-- that's all... 
END;
