--
-- ADVance MS model for electromechanical harvesting_resonator adms_vhdlams_ESYCOM_juine2007
--
-- The model is adapted for the ESIEE device of 2011
LIBRARY DISCIPLINES;
USE DISCIPLINES.ELECTROMAGNETIC_SYSTEM.ALL;
library IEEE;
use IEEE.MATH_REAL.all;



ENTITY ESIEE_II_electrostatic_transducer IS
	GENERIC(
          d:real:=1.0e-6; --transducer's gap
          d_stoppers:real:=1.0e-6; -- stopper position

          l:real:=1.0e-2; --transducer's length
          w:real:=400.0e-6; --transducer's width
          N: integer -- number of capacitors
		);

     -- les terminaux : C1 et C2 are the capacitor's electrodes 
     --                 Mp et Mm are the mechanical terminal
     --                 generating the transducer's force (voltage)  
     --                 Their polarity is important
     --                 pos: the x-coordinate of the transducer, 
     --                       an output termninal for measurement purposes
	PORT (
		TERMINAL C1        : Electrical ;
          TERMINAL C2      : Electrical ;
		TERMINAL Mp     : Electrical ;
		TERMINAL Mm         : Electrical; 
          TERMINAL pos          : Electrical;
          TERMINAL capa          : Electrical
	);
END;

ARCHITECTURE gap_closing_transducer OF ESIEE_II_electrostatic_transducer IS

function transducer_capacitance_gap_closing(x:real; d:real;S:real) return real is
variable capa:real;
begin
	capa:=8.85e-12*S/(d-x);
return capa;
end;

function transducer_capacitance_gap_closing_symetrical(x:real; N: integer; d:real;l:real;h:real) return real is
variable capa:real;
constant a:real:=5.0/380.0;
begin
	--capa:=8.85e-12*real(N)*S*2.0*d/(d**2.0-x**2.0); // parois droites
  -- Parois obliques:
     capa:=8.85e-12*real(N)*l/(2.0*a)*log((d-x+2.0*h*a)*(d+x+2.0*h*a)/(d*d-x*x));
return capa;
end;


quantity charge : real:=0.0;
quantity dCvar_dx : real;
quantity uC across iC through C1 to C2; -- the capacitor voltage
quantity force across v through Mp to Mm; -- the force of transducer 
quantity x across ix through pos to electrical_ground; -- position of the mobile electrode of transducer 
quantity Cvar across icapa through capa to electrical_ground;  
quantity F_stoppers:real; -- position of the mobile electrode of transducer 

constant mu_stoppers:real:=4.0e0;
constant k_stoppers:real:=4.0e5;
constant dx :real:=0.001e-6;
BEGIN

-- mechanical side model : a force generator. 
-- However, the position must be known for the force calculation
-- We integrate the speed (the current through Mp and Mm) to know the position 
v==-x'dot;
-- then we calculate the capacitance gradient  at x 
dCvar_dx==(transducer_capacitance_gap_closing_symetrical(x+dx,N,  d, l, w)-transducer_capacitance_gap_closing_symetrical(x, N, d, l, w))/dx;
-- and the force

if (x'above(d-d_stoppers) or  not (x'above(-d+d_stoppers)))use
 F_stoppers==mu_stoppers*v-k_stoppers*(abs(x)-(d-d_stoppers))*sign(x);
 else
 F_stoppers==0.0;
end use;
force==uC*uC*dCvar_dx/2.0+F_stoppers;

-- Electrical side model : a simple capacitor
-- first we calculate the capacitance value at x
Cvar==transducer_capacitance_gap_closing_symetrical(x, N, d, l, w); -- le dispo de l'ESIEE du 2011
-- then the charge 
ic==charge'dot;
-- then the capacitor voltage
uC==charge/Cvar;

-- that's all... 
break on X'above(d-d_stoppers);
break on X'above(-d+d_stoppers);
END;

